////////////////////////////////////////////////////////////////////////////
//	FADEBLCK.CPP		show a bitmap, and make it disappear by
//									random permutation.
////////////////////////////////////////////////////////////////////////////

#include <owl\applicat.h>
#include <owl\framewin.h>
#include <owl\dc.h>
#include <owl\gdiobjec.h>
#include <owl\color.h>
#pragma hdrstop
#include "fadeblck.rh"

#define BITMAPFILE "mask.bmp"

const int BITMAP_W = 128;			//bitmap width
const int BITMAP_H = 96;      //bitmap height

/*the size of the array has to be a prime number, but not all prime numbers
	will work. 128 * 96 is 12288, but that is not a prime number, so I chose
	the next prime number that works which is 12301. The index will be
	from 0 to 12300. */
const int NUMPIXELS = 12300;	//128 * 96 (plus some change)

///////////////////////////////////////////////////////////////////////////
class TFadeWin: public TFrameWindow {
public:
	TFadeWin(TWindow* parent, const char far* title);
	~TFadeWin();
protected:
	void SetupWindow();
	void GetWindowClass(WNDCLASS& wndClass);
	char far* GetClassName() {return "TFadeWin";}
	void Paint(TDC& dc, BOOL erase, TRect& rect);
	void EvSize(UINT sizeType, TSize& size);
private:
	void CenterBitmap();
	void CmFadeMask();
	void CmShowMask();
	void CmEnableFadeMask(TCommandEnabler &commandEnabler);
	void CmEnableShowMask(TCommandEnabler &commandEnabler);

	TBitmap *mask;
	BOOL ShowMask;		//show bitmap if true
	int maskX, maskY;
	TPalette *palette;
DECLARE_RESPONSE_TABLE(TFadeWin);
};

DEFINE_RESPONSE_TABLE1(TFadeWin, TFrameWindow)
	EV_WM_SIZE,
	EV_COMMAND(CM_FADEMASK, CmFadeMask),
	EV_COMMAND(CM_SHOWMASK, CmShowMask),
	EV_COMMAND_ENABLE(CM_FADEMASK, CmEnableFadeMask),
	EV_COMMAND_ENABLE(CM_SHOWMASK, CmEnableShowMask),
END_RESPONSE_TABLE;


TFadeWin::TFadeWin(TWindow* parent, const char far* title)
	: TFrameWindow(parent, title),
		TWindow(parent, title),
		ShowMask(TRUE)
{
	AssignMenu(ID_MENU);
//*
	TDib *dib = new TDib(GetApplicationObject()->GetInstance(),
		BITMAP_MASK);											//make a dib from resource
	palette = new TPalette(*dib);				//make a palette from dib
	mask = new TBitmap(*dib, palette);	//bitmap with given palette
	delete dib;
//*/
	Attr.X = 50;
	Attr.Y = 50;
	Attr.W = 400;
	Attr.H = 250;

/*	you can read bitmap from disk also
	TDib *dib = new TDib(BITMAPFILE);
	mask = new TBitmap(*dib);
	palette = new TPalette(*dib);
	delete dib;
*/
}


TFadeWin::~TFadeWin()
{
	delete mask;
	delete palette;
}


void TFadeWin::SetupWindow()
{
	TWindow::SetupWindow();
	CenterBitmap();
}


void TFadeWin::CenterBitmap()
{
	TRect rect = GetClientRect();
	maskX = (rect.Width() - BITMAP_W) / 2;
	maskY = (rect.Height() - BITMAP_H) / 2;
}


void TFadeWin::EvSize(UINT sizeType, TSize& size)
{
	TFrameWindow::EvSize(sizeType, size);
	CenterBitmap();
	Invalidate();
	UpdateWindow();		//not strictly necessary
}


void TFadeWin::GetWindowClass(WNDCLASS& wndClass)
{
	TFrameWindow::GetWindowClass(wndClass);
	wndClass.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH);
}


void TFadeWin::Paint(TDC& dc, BOOL /*erase*/, TRect& /*rect*/)
{
	if (ShowMask) {						//only if we need to draw mask
		dc.SelectObject(*palette);
		dc.RealizePalette();

		TMemoryDC memdc(dc);
		memdc.SelectObject(*mask);
		memdc.SelectObject(*palette);

		dc.BitBlt(
			maskX, maskY,          // Destination X,Y coordinate
			BITMAP_W, BITMAP_H,    // Bitmap width and height
			memdc,                 // Source device context
			0, 0,                  // Source X,Y coordinate
			SRCCOPY);              // Raster display operation

		memdc.RestoreBitmap();   // Not strictly necessary
		memdc.RestorePalette();
		dc.RestorePalette();
	}
}


void TFadeWin::CmFadeMask()
{
	ShowMask = FALSE;			//no more redrawing
	TClientDC *clientDC;
	clientDC = new TClientDC(HWindow);

	int index = 1;		//random permutation value
	int x, y;					//coordinate for black dot
	for (int i = 0; i < NUMPIXELS; i++) {
		index <<= 1;										//multiply by 2
		index %= (NUMPIXELS + 1);   		//and take the remainder

		if (index <= 12288) {						//we don't care indexes 12289~12300
			x = (index - 1) % BITMAP_W;
			y = (index - 1) / BITMAP_W;
			clientDC->SetPixel(x + maskX, y + maskY, TColor::Black);
		}
	}
	delete clientDC;
}


void TFadeWin::CmShowMask()
{
	ShowMask = TRUE;
	TClientDC *clientDC;
	clientDC = new TClientDC(HWindow);
	clientDC->SelectObject(*palette);
	clientDC->RealizePalette();

	TMemoryDC memdc(*clientDC);
	memdc.SelectObject(*mask);
	memdc.SelectObject(*palette);

	int index = 1;		//random permutation value
	int x, y;					//coordinate for black dot
	for (int i = 0; i < NUMPIXELS; i++) {
		index <<= 1;										//multiply by 2
		index %= (NUMPIXELS + 1);   		//and take the remainder

		if (index <= 12288) {						//we don't care indexes 12289~12300
			x = (index - 1) % BITMAP_W;
			y = (index - 1) / BITMAP_W;
//			clientDC->SetPixel(x + maskX, y + maskY, memdc.GetPixel(x, y));
			clientDC->BitBlt(x + maskX, y + maskY, 1, 1, memdc, x, y, SRCCOPY);
		}
	}
	memdc.RestorePalette();
	clientDC->RestorePalette();
	delete clientDC;
}


void TFadeWin::CmEnableFadeMask(TCommandEnabler &commandEnabler)
{
	commandEnabler.Enable(ShowMask);
}

void TFadeWin::CmEnableShowMask(TCommandEnabler &commandEnabler)
{
	commandEnabler.Enable(!ShowMask);
//	commandEnabler.Enable(TRUE);
}

///////////////////////////////////////////////////////////////////////////
class TFadeApp: public TApplication {
public:
	TFadeApp(const char far* name)
		: TApplication(name) {};
	void InitMainWindow();
};


void TFadeApp::InitMainWindow()
{
	MainWindow = new TFadeWin(0, "Fade to Black");
	MainWindow->SetIcon(this, ID_ICON);
}


int OwlMain(int /*argc*/, char* /*argv*/[])
{
	TFadeApp app("Fade to Black");
	return app.Run();
}

